@@ -11,7 +11,7 @@ use crate::queues::mpsc::jiffy;
 use super::{record::Record, retire_node::RetireNode, Guard};
 
 /// A Thread-Local instance to interact with a single Hazard-Pointer-Domain
-pub struct Domain {
+pub struct TLDomain {
     /// The Refernce to the Shared-Global State for the Hazard-Pointer-Domain
     global: Arc<DomainGlobal>,
 
@@ -24,28 +24,32 @@ pub struct Domain {
     /// The List of Memory-Nodes marked as being ready to retire, by the
     /// algorithm, that have not yet been reclaimed and may still be in use
     /// by some other Part of the overall system
-    r_list: Vec<RetireNode<()>>,
+    r_list: Vec<RetireNode>,
 }
 
-impl Debug for Domain {
+unsafe impl Send for TLDomain {}
+
+impl Debug for TLDomain {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         write!(f, "Domain ()")
     }
 }
 
-impl Drop for Domain {
+impl Drop for TLDomain {
     fn drop(&mut self) {
+        // This will attempt to retire/reclaim all the waiting entries for the
+        // local Thread. Although this is not garantued to free all the marked
+        // Nodes, because other Threads might still use the Data, in that case
+        // we will leak the memory at the moment
+        self.reclaim();
+
         // TODO
-        // This should (at least try to) reclaim all the current
-        // Elements in the R-List, as this would otherwise result
-        // in a memory leak
-        for r_entry in self.r_list.drain(..) {
-            r_entry.retire();
-        }
+        // Figure out what to do with the remaining Data that could not be
+        // retired in this instance
     }
 }
 
-impl Domain {
+impl TLDomain {
     /// Creates a new Domain with the given shared Global and reclaim Threshold
     pub fn new(global: Arc<DomainGlobal>, reclaim_threshold: usize) -> Self {
         let (rx, tx) = jiffy::queue();
@@ -96,10 +100,16 @@ impl Domain {
         let tmplist = std::mem::take(&mut self.r_list);
 
         for node in tmplist {
-            if plist.contains(&(node.ptr as *const ())) {
+            if plist.contains(&node.const_ptr()) {
                 self.r_list.push(node);
             } else {
-                node.retire();
+                // # Safety
+                // This is safe because we have read all the current Hazard-
+                // Pointers and no one is protecting this Ptr, meaning that
+                // the Data is not accessed by another Thread at the same
+                // time and will also not be accessed in the Future and is
+                // therefore save to retire.
+                unsafe { node.retire() };
             }
         }
     }
@@ -125,11 +135,7 @@ impl Domain {
             _ => self.generate_new_record(),
         };
 
-        Guard {
-            inner: std::ptr::null_mut(),
-            record: record_ptr,
-            record_returner: self.record_sender.clone(),
-        }
+        Guard::new(std::ptr::null_mut(), record_ptr, self.record_sender.clone())
     }
 
     /// Loads the most recent Ptr-Value from the given AtomicPtr, protects it
@@ -140,16 +146,7 @@ impl Domain {
         atom_ptr: &atomic::AtomicPtr<T>,
         load_order: atomic::Ordering,
     ) -> Guard<T> {
-        let record_ptr = match self.record_receiver.try_dequeue() {
-            Ok(r) => r,
-            _ => self.generate_new_record(),
-        };
-
-        let mut guard: Guard<T> = Guard {
-            inner: std::ptr::null_mut(),
-            record: record_ptr,
-            record_returner: self.record_sender.clone(),
-        };
+        let mut guard: Guard<T> = self.empty_guard();
 
         guard.protect(atom_ptr, load_order);
 
