@@ -1,4 +1,4 @@
-use std::{collections::HashSet, fmt::Debug, mem::ManuallyDrop, sync::atomic};
+use std::{collections::HashSet, fmt::Debug, sync::atomic};
 
 use crate::hazard_ptr::Record;
 
@@ -25,34 +25,35 @@ impl DomainGlobal {
 
     /// Checks all the current Hazard-Pointers and returns a Set of all
     /// currently protected PTRs stored in them
-    pub(crate) fn get_protections(&self) -> HashSet<*const ()> {
+    pub fn get_protections(&self) -> HashSet<*const ()> {
         let mut plist = HashSet::new();
 
         let ptr = self.records.load(atomic::Ordering::SeqCst);
         if ptr.is_null() {
             return plist;
         }
-        let mut current_item = ManuallyDrop::new(unsafe { Box::from_raw(ptr) });
+
+        let mut current = unsafe { &*ptr };
 
         loop {
-            let ptr_val = current_item.ptr.load(atomic::Ordering::SeqCst);
+            let ptr_val = current.ptr.load(atomic::Ordering::SeqCst);
             if !ptr_val.is_null() {
                 plist.insert(ptr_val as *const ());
             }
 
-            match current_item.load_next(atomic::Ordering::SeqCst) {
-                Some(i) => {
-                    current_item = i;
-                }
-                None => break,
-            };
+            let next_ptr = current.next.load(atomic::Ordering::SeqCst);
+            if next_ptr.is_null() {
+                break;
+            }
+
+            current = unsafe { &*next_ptr };
         }
 
         plist
     }
 
     /// This is used to add a new Record to the End of the Hazard-Pointer-List
-    pub(crate) fn append_record(&self, n_record_ptr: *mut Record<()>) {
+    pub fn append_record(&self, n_record_ptr: *mut Record<()>) {
         let ptr = self.records.load(atomic::Ordering::SeqCst);
         if ptr.is_null()
             && self
@@ -70,7 +71,7 @@ impl DomainGlobal {
 
         let mut last_record = {
             let ptr = self.records.load(atomic::Ordering::SeqCst);
-            ManuallyDrop::new(unsafe { Box::from_raw(ptr) })
+            unsafe { &*ptr }
         };
         loop {
             match last_record.load_next(atomic::Ordering::SeqCst) {
@@ -95,10 +96,71 @@ impl DomainGlobal {
             };
         }
     }
+
+    fn clean_up(&mut self) {
+        let current_ptr = self.records.load(atomic::Ordering::SeqCst);
+        if current_ptr.is_null() {
+            return;
+        }
+
+        let mut current = unsafe { Box::from_raw(current_ptr) };
+        loop {
+            let next_ptr = current.next.load(atomic::Ordering::SeqCst);
+            drop(current);
+
+            if next_ptr.is_null() {
+                return;
+            }
+            current = unsafe { Box::from_raw(next_ptr) };
+        }
+    }
 }
 
 impl Drop for DomainGlobal {
     fn drop(&mut self) {
-        println!("Dropped Global");
+        self.clean_up();
+    }
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn append_load_hazards() {
+        let global = DomainGlobal::new();
+
+        {
+            let expected = HashSet::new();
+            assert_eq!(expected, global.get_protections());
+        }
+
+        let record_ptr = Box::into_raw(Record::<u64>::boxed_empty());
+        global.append_record(record_ptr as *mut Record<()>);
+
+        {
+            let expected = HashSet::new();
+            assert_eq!(expected, global.get_protections());
+        }
+
+        let record = unsafe { &*record_ptr };
+        record
+            .ptr
+            .store(0x123 as *mut u64, atomic::Ordering::SeqCst);
+
+        {
+            let mut expected = HashSet::new();
+            expected.insert(0x123 as *const ());
+            assert_eq!(expected, global.get_protections());
+        }
+
+        record
+            .ptr
+            .store(std::ptr::null_mut(), atomic::Ordering::SeqCst);
+
+        {
+            let expected = HashSet::new();
+            assert_eq!(expected, global.get_protections());
+        }
     }
 }
