@@ -36,7 +36,7 @@ mod async_queue;
 #[cfg(feature = "async")]
 pub use async_queue::*;
 
-use super::{DequeueError, EnqueueError};
+use crate::queues::{DequeueError, EnqueueError};
 
 /// One of the Sender, created by calling [`queue`]
 pub struct Sender<T> {
@@ -132,10 +132,10 @@ impl<T> Sender<T> {
         // Load our target absolute position, on where to insert the next
         // Element
         //
-        // This needs to use Ordering::SeqCst because we would otherwise have
-        // one half of the load-store operation be Ordering::Relaxed, which
+        // This needs to use at least Ordering::AcqRel because we would otherwise
+        // have one half of the load-store operation be Ordering::Relaxed, which
         // is not what we need
-        let location = self.tail.fetch_add(1, atomic::Ordering::SeqCst);
+        let location = self.tail.fetch_add(1, atomic::Ordering::AcqRel);
 
         // Get the current tail-buffer, where we would initially attempt to
         // insert the Element into
@@ -152,16 +152,9 @@ impl<T> Sender<T> {
         // actually contains our Target-Location, because the buffer we
         // find could come after the Buffer that we actually need
         while location >= end {
-            // If the currently loaded Buffer has no next Ptr, meaning
-            // it is currently the last Buffer in the Queue, we need to
-            // create a new Buffer and append it
-            if tmp_buffer.next.load(atomic::Ordering::Acquire).is_null() {
-                // Attempt to allocate a new Buffer
-                tmp_buffer.allocate_next(tmp_buffer_ptr, &self.tail_of_queue);
-            }
-
-            // Load the new Tail of the Queue
-            tmp_buffer_ptr = self.tail_of_queue.load(atomic::Ordering::Acquire);
+            // Move to the next Buffer in the Queue, this will also automatically create
+            // a new Buffer if there is no next Buffer currently available
+            tmp_buffer_ptr = tmp_buffer.go_to_next(tmp_buffer_ptr, &self.tail_of_queue);
             tmp_buffer = unsafe { &*tmp_buffer_ptr };
 
             // Recalculate the current End of the new Tail-Buffer
@@ -285,7 +278,7 @@ impl<T> Receiver<T> {
     /// # Example
     /// ```
     /// # use nolock::queues::mpsc::jiffy;
-    /// # use nolock::queues::mpsc::DequeueError;
+    /// # use nolock::queues::DequeueError;
     ///
     /// let (mut rx, tx) = jiffy::queue::<usize>();
     ///
@@ -297,7 +290,7 @@ impl<T> Receiver<T> {
     /// // Attempt to get the next Element, but there is none so we get
     /// // the right Error indicating that there is no Element to dequeue at
     /// // that moment
-    /// assert_eq!(Err(DequeueError::WouldBlock), rx.try_dequeue());
+    /// assert_eq!(Err(DequeueError::Empty), rx.try_dequeue());
     /// ```
     pub fn try_dequeue(&mut self) -> Result<T, DequeueError> {
         // Loads the current Buffer that should be used
@@ -324,7 +317,7 @@ impl<T> Receiver<T> {
                 // currently nothing to load
                 match current_queue.buffer.get(current_queue.head) {
                     Some(n) => n,
-                    None => return Err(DequeueError::WouldBlock),
+                    None => return Err(DequeueError::Empty),
                 }
             }
         };
@@ -334,7 +327,7 @@ impl<T> Receiver<T> {
             current_queue.head += 1;
 
             if !self.move_to_next_buffer() {
-                return Err(DequeueError::WouldBlock);
+                return Err(DequeueError::Empty);
             }
 
             current_queue = unsafe { &mut *self.head_of_queue };
@@ -345,7 +338,7 @@ impl<T> Receiver<T> {
                     current_queue = unsafe { &mut *self.head_of_queue };
                     match current_queue.buffer.get(current_queue.head) {
                         Some(t) => t,
-                        None => return Err(DequeueError::WouldBlock),
+                        None => return Err(DequeueError::Empty),
                     }
                 }
             };
@@ -409,7 +402,7 @@ impl<T> Receiver<T> {
                                     None => return Err(DequeueError::Closed),
                                 }
                             } else {
-                                return Err(DequeueError::WouldBlock);
+                                return Err(DequeueError::Empty);
                             }
                         }
                     };
@@ -419,7 +412,7 @@ impl<T> Receiver<T> {
                 // Try to load the found Node
                 let tmp_n = match tmp_head_of_queue.buffer.get(tmp_head) {
                     Some(n) => n,
-                    None => return Err(DequeueError::WouldBlock),
+                    None => return Err(DequeueError::Empty),
                 };
 
                 // Actually load the Data from the Node
@@ -453,7 +446,7 @@ impl<T> Receiver<T> {
                 Some(data)
                 */
             }
-            _ => Err(DequeueError::WouldBlock),
+            _ => Err(DequeueError::Empty),
         }
     }
 
@@ -475,7 +468,7 @@ impl<T> Receiver<T> {
                 Err(e) => match e {
                     // If we had a simply error telling us that there is item
                     // in the Queue, we should simply continue
-                    DequeueError::WouldBlock => {}
+                    DequeueError::Empty => {}
                     // If the Queue has been closed, there is nothing we could
                     // retrieve in the Future and therefore we return None
                     DequeueError::Closed => return None,
@@ -529,7 +522,20 @@ impl<T> Debug for Receiver<T> {
 
 impl<T> Drop for Receiver<T> {
     fn drop(&mut self) {
-        close_side(&self.closed, || self.head_of_queue as *mut BufferList<T>);
+        close_side(&self.closed, || {
+            let mut current_ptr = self.head_of_queue;
+            let mut current = unsafe { &*current_ptr };
+
+            loop {
+                let next_ptr = current.next.load(atomic::Ordering::SeqCst);
+                if next_ptr.is_null() {
+                    return current_ptr;
+                }
+
+                current_ptr = next_ptr;
+                current = unsafe { &*current_ptr };
+            }
+        });
     }
 }
 
@@ -564,7 +570,7 @@ mod tests {
     fn dequeue_empty() {
         let (mut rx, tx) = queue::<u8>();
 
-        assert_eq!(Err(DequeueError::WouldBlock), rx.try_dequeue());
+        assert_eq!(Err(DequeueError::Empty), rx.try_dequeue());
         drop(tx);
     }
 
@@ -640,6 +646,18 @@ mod tests {
         assert_eq!(Err(DequeueError::Closed), rx.try_dequeue());
     }
 
+    #[test]
+    fn enqueue_some_close() {
+        let (rx, tx) = queue::<usize>();
+
+        for index in 0..10 {
+            tx.enqueue(index).unwrap();
+        }
+
+        drop(tx);
+        drop(rx);
+    }
+
     #[test]
     fn iter_mut() {
         let (mut rx, tx) = queue::<usize>();
